#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const createApplication_util_1 = require("./src/createApplication.util");
const createComponentFiles_util_1 = require("./src/createComponentFiles.util");
const applicationConfig_1 = require("./src/applicationConfig");
const ora = require('ora');
const fs = require('fs');
let [, , configFilePath] = process.argv;
const defaultPreducksConfig = {
    interfaces: {},
    reducers: {
        DEFAULT: {
            store: {},
            actions: {},
        },
    },
};
const configPath = configFilePath ? `${process.cwd()}/` + configFilePath : `${process.cwd()}/preducksConfig.json`;
if (fs.existsSync(configPath)) {
    fs.readFile(configPath, (err, data) => {
        if (err) {
            console.log('\x1b[31m', 'error reading preducksConfig: ', err);
        }
        else {
            const preducksConfig = JSON.parse(data);
            createPreducksApp(preducksConfig.appName ? preducksConfig.appName : 'preducksApp', preducksConfig);
        }
    });
}
else {
    console.log('\x1b[35m', '  ❗️   no preducks JSON config found. cooking up a default preducks app ✨', "\x1b[0m");
    createPreducksApp('preducksApp', defaultPreducksConfig);
}
function createPreducksApp(appName, preducksConfig) {
    createApplication_util_1.default({ path: process.cwd(),
        appName: appName,
        genOption: 1,
        storeConfig: preducksConfig });
    createComponentFiles_util_1.default(applicationConfig_1.default.components, preducksConfig, process.cwd(), appName, true);
}
// console.log('🦆 building preducks app 🦆');
const throbber = ora({
    text: "\x1b[36m" + 'building preducks app',
    spinner: {
        frames: ['   🥚  ', '   🐣  ', '   🐤  ', '   🐥  '],
        interval: 250,
    },
}).start();
setTimeout(() => {
    throbber.stopAndPersist({
        symbol: '   🦆  ',
        text: "\x1b[42m" + "\x1b[30m" + 'finished building preducks app' + "\x1b[0m" + '  🦆  ',
    });
}, 1000);
//# sourceMappingURL=index.js.map